@@ -26,10 +26,16 @@ import {
   IProgress,
   CiteProc,
   IUnambiguousItemIdentifier,
-  ICitableItemRecordsBySource
+  ICitableItemRecordsBySource,
+  IAlternativeFormat
 } from './types';
 import { zoteroPlugin } from './zotero';
-import { DefaultMap, harmonizeData, simpleRequest } from './utils';
+import {
+  DefaultMap,
+  generateRandomID,
+  harmonizeData,
+  simpleRequest
+} from './utils';
 
 import * as CSL from 'citeproc';
 import { CitationSelector } from './components/citationSelector';
@@ -39,7 +45,12 @@ import {
   TranslationBundle
 } from '@jupyterlab/translation';
 import { NotebookAdapter, NotebookButtons } from './adapters/notebook';
-import { ICommandPalette } from '@jupyterlab/apputils';
+import {
+  Dialog,
+  ICommandPalette,
+  showDialog,
+  showErrorMessage
+} from '@jupyterlab/apputils';
 import { fetchAPI, requestAPI } from './handler';
 import { StyleSelector } from './components/styleSelector';
 import {
@@ -55,6 +66,9 @@ import { UpdateProgress } from './components/progressbar';
 import { Signal } from '@lumino/signaling';
 import { URLExt } from '@jupyterlab/coreutils';
 import { refreshIcon } from '@jupyterlab/ui-components';
+import OutputMode = CiteProc.OutputMode;
+import { cite2cPlugin } from './formats/cite2c';
+import { markdownDOIPlugin } from './formats/markdownDOI';
 
 const PLUGIN_ID = 'jupyterlab-citation-manager:plugin';
 
@@ -120,9 +134,11 @@ class UnifiedCitationManager implements ICitationManager {
   private localeCache: Map<string, string>;
   protected defaultStyleID = 'apa.csl';
   protected allReady: ICancellablePromise<any>;
+  protected formats: IAlternativeFormat<any>[];
 
   progress: Signal<UnifiedCitationManager, IProgress>;
   private currentAdapter: IDocumentAdapter<any> | null = null;
+  private outputFormat: CiteProc.OutputMode = 'html';
 
   protected createAllReadyPromiseWrapper(): ICancellablePromise<any> {
     const REASON_CANCELLED = 'cancelled';
@@ -165,6 +181,7 @@ class UnifiedCitationManager implements ICitationManager {
     this.progress = new Signal(this);
     this.styles = new StylesManager(trans, this);
     this.selector = new CitationSelector(trans);
+    this.formats = [];
     this.selector.hide();
     this.adapters = new WeakMap();
     this.processors = new WeakMap();
@@ -182,13 +199,7 @@ class UnifiedCitationManager implements ICitationManager {
       });
 
       panel.context.ready.then(() => {
-        const adapter = this.getAdapter(panel);
-        adapter.migrateFormat().then(async migrated => {
-          if (migrated) {
-            console.log('Migrated citations, will refresh now...');
-            await this.processFromScratch(panel);
-          }
-        });
+        this.offerMigration(panel).catch(console.warn);
       });
 
       panel.context.saveState.connect((sender, state) => {
@@ -216,6 +227,69 @@ class UnifiedCitationManager implements ICitationManager {
     this.allReady = this.createAllReadyPromiseWrapper();
   }
 
+  registerFormat(format: IAlternativeFormat<any>) {
+    this.formats.push(format);
+    console.log(`${format.name} format registered`);
+    if (this.notebookTracker.currentWidget) {
+      this.offerMigration(this.notebookTracker.currentWidget).catch(
+        console.warn
+      );
+    }
+  }
+
+  async offerMigration(panel: NotebookPanel) {
+    const adapter = this.getAdapter(panel);
+    for (const format of this.formats) {
+      const detectionResult = format.detect(panel, adapter);
+      if (
+        detectionResult.citationsDetected !== 0 ||
+        detectionResult.bibliographiesDetected !== 0
+      ) {
+        const migrateButton = Dialog.okButton({
+          label: this.trans.__('Migrate')
+        });
+        const decision = await showDialog({
+          title: this.trans.__(
+            'Importing %1 citations is possible',
+            format.name
+          ),
+          body: this.trans.__(
+            'Detected %1 citation in %2 format. Would you like to migrate these citations to Citation Manager format?',
+            detectionResult.citationsDetected,
+            format.name
+          ),
+          buttons: [Dialog.cancelButton(), migrateButton],
+          defaultButton: 1
+        });
+        if (decision.button === migrateButton) {
+          const result = await format.migrateFrom(panel, adapter);
+          if (result.aborted) {
+            await showErrorMessage(
+              this.trans.__('Migration from %1 failed', format.name),
+              result.message
+            );
+          }
+          if (result.failures.length !== 0) {
+            await showErrorMessage(
+              this.trans.__('Migration from %1 was not complete', format.name),
+              this.trans.__(
+                'There were %1 failures: ',
+                result.failures.length
+              ) +
+                JSON.stringify(result.failures) +
+                '\n\n' +
+                result.message
+            );
+          }
+          console.log(
+            `${result.migratedCitationsCount} citations migrated from ${format.name}`
+          );
+          await this.processFromScratch(panel);
+        }
+      }
+    }
+  }
+
   async previewStyle(
     style: IStyle,
     maxCitations: number
@@ -326,6 +400,7 @@ class UnifiedCitationManager implements ICitationManager {
 
   protected updateSettings(settings: ISettingRegistry.ISettings) {
     this.defaultStyleID = settings.get('defaultStyle').composite as string;
+    this.outputFormat = settings.get('outputFormat').composite as OutputMode;
   }
 
   public registerReferenceProvider(provider: IReferenceProvider): void {
@@ -426,16 +501,6 @@ class UnifiedCitationManager implements ICitationManager {
     return options;
   }
 
-  private _generateRandomID(existingIDs: Set<string>): string {
-    let isUnique = false;
-    let id = '';
-    while (!isUnique) {
-      id = Math.random().toString(36).slice(-5);
-      isUnique = !existingIDs.has(id);
-    }
-    return id;
-  }
-
   private getAdapter(content: NotebookPanel): IDocumentAdapter<any> {
     let adapter = this.adapters.get(content);
     if (!adapter) {
@@ -524,7 +589,7 @@ class UnifiedCitationManager implements ICitationManager {
           ...citationsAfter.map(c => c.citationId)
         ]);
 
-        const newCitationID = this._generateRandomID(existingCitationIDs);
+        const newCitationID = generateRandomID(existingCitationIDs);
 
         const citationsBeforeMap: Record<number, ICitation> =
           Object.fromEntries(citationsBefore.map((c, index) => [index, c]));
@@ -650,6 +715,10 @@ class UnifiedCitationManager implements ICitationManager {
           const styleAsText = result.response.responseText;
           return new CSL.Engine(this, styleAsText);
         });
+      })
+      .then((engine: CiteProc.IEngine) => {
+        engine.setOutputFormat(this.outputFormat);
+        return engine;
       });
   }
 
@@ -884,6 +953,8 @@ const managerPlugin: JupyterFrontEndPlugin<ICitationManager> = {
 const plugins: JupyterFrontEndPlugin<any>[] = [
   managerPlugin,
   zoteroPlugin,
+  cite2cPlugin,
+  markdownDOIPlugin,
   openerPlugin
 ];
 
